----------------------------------------------------------------------------------
-- COPYRIGHT 2019 Jess Eduardo Mndez Rosales
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--
--							LIBRERA DISPLAY 7 SEGMENTOS
--
-- Descripcin: Con sta librera podrs controlar hasta 8 Displays que comparten
-- 	un mismo bus de datos para cada segmento includo el punto decimal.
--
-- Caractersticas:
-- 
--	1) Frecuencia de reloj configurable.
-- 2) Numero de Display configurable. Se puede controlar mximo 8 Displays.
-- 3) Se puede seleccionar nodo o catodo comn.
-- 4) Se puede seleccionar el estado de activacin del transistor.
-- 5) Conversin binaria a decimal utilizando el algoritmo Shift and Add 3
-- 6) Multiplexeo de los Display de 1ms.
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_unsigned.ALL;

entity LIB_DISPLAY7SEG_RevB is

GENERIC(
			FPGA_CLK 		  : INTEGER   := 100_000_000; -- Frecuencia de reloj.
			NUM_DISPLAY		  : INTEGER	  := 5;			 	-- Nmero de Displays a utilizar. Mximo 8.
			SEL_ANODO_CATODO : STD_LOGIC := '0';			-- Selector de nodo ('0') o ctodo ('1').
			SEL_TRANSISTOR   : STD_LOGIC := '0'				-- Selector de transistor, activo en alto '1' o activo en bajo '0'.

);

PORT(
		CLK 		: IN  STD_LOGIC;								-- Reloj FPGA.
		NUM_BIN  : IN  STD_LOGIC_VECTOR(26 DOWNTO 0);	-- Numero binario a convertir.
		PUNTOS 	: IN  STD_LOGIC_VECTOR(7  DOWNTO 0);	-- Activa los puntos decimales.
		DISPLAY 	: OUT STD_LOGIC_VECTOR(7  DOWNTO 0);	-- Puerto conectado a los segmentos del display 7-A, 6-B, 5-C, ... 0-PD.
		T			: OUT STD_LOGIC_VECTOR(7  DOWNTO 0) 	-- Puerto conectado a los transistores.
);


end LIB_DISPLAY7SEG_RevB;

architecture Behavioral of LIB_DISPLAY7SEG_RevB is

-------------------------------------------------------------------
-- Declaracin del componente que convierte de binario a decimal --

COMPONENT BINARIO_A_DECIMAL is

PORT(
		DATO_BINARIO : IN  STD_LOGIC_VECTOR(26 DOWNTO 0);
		DATO_DECIMAL : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);

end COMPONENT BINARIO_A_DECIMAL;

-------------------------------------------------------------------
-------------------------------------------------------------------


CONSTANT ESCALA_MUX : INTEGER := (FPGA_CLK/1000); -- Constante que se utiliza para generar el retardo de 1ms

signal dato_decimal : std_logic_vector(31 downto 0) := (others => '0'); -- Sea que contiene al valor decimal ya convertido.
signal conta_mux 	  : integer range 0 to ESCALA_MUX := 0;					-- Contador para retardo de 1ms.
signal edo	 		  : integer range 0 to 10 := 0;								-- Seal para mquina de estados.
signal edo_mux 	  : integer range 0 to 7 := 0;								-- Seal para mquina de estados para el multiplexeo.
signal num_aux 	  : integer range 0 to 9 := 0;								-- Seal auxiliar que se decodificar.
signal punto 	     : std_logic := '0';											-- Seal para activar o desactivar el punto decimal.
signal b 			  : std_logic := '0';											-- Bandera que indica cuando transcurrin el retardo de 1ms.

begin

-------------------------------------------------
-- Instancia del convertidor binario a decimal --

convertidor: BINARIO_A_DECIMAL
PORT map( NUM_BIN, DATO_DECIMAL );

-------------------------------------------------
-------------------------------------------------


------------------------------------------
-- Proceso que genera el retardo de 1ms --

process(clk)
begin
if rising_edge(cLK) then
	conta_mux <= conta_mux+1;
	if conta_mux = ESCALA_MUX then
		conta_mux <= 0;
		b <= '1';
	else
		b <= '0';
	end if;
end if;
end process;

------------------------------------------
------------------------------------------


--------------------------------------------------
-- Mquina de estados que genera el multiplexeo --

process(CLK)
begin
if rising_edge(CLK) then
	if 	edo_mux = 0 then
		if SEL_TRANSISTOR = '1' then
			T <= "00000001";
		else
			T <= "11111110";
		end if;
		
		if b = '1' then			
			if num_display = 1 then
				edo_mux <= 0;
			else
				edo_mux <= 1;
			end if;
		end if;
		
	elsif edo_mux = 1 then
		if SEL_TRANSISTOR = '1' then
			T <= "00000010";
		else
			T <= "11111101";
		end if;	
			
		if b = '1' then
			if num_display = 2 then
				edo_mux <= 0;
			else
				edo_mux <= 2;
			end if;
		end if;
			
	elsif edo_mux = 2 then
		if SEL_TRANSISTOR = '1' then
			T <= "00000100";
		else
			T <= "11111011";
		end if;	

		if b = '1' then
			if num_display = 3 then
				edo_mux <= 0;
			else
				edo_mux <= 3;
			end if;
		end if;
			
	elsif edo_mux = 3 then
		if SEL_TRANSISTOR = '1' then
			T <= "00001000";
		else
			T <= "11110111";
		end if;
			
		if b = '1' then
			if num_display = 4 then
				edo_mux <= 0;
			else
				edo_mux <= 4;
			end if;
		end if;
		
	elsif edo_mux = 4 then
		if SEL_TRANSISTOR = '1' then
			T <= "00010000";
		else
			T <= "11101111";
		end if;
		
		if b = '1' then
			if num_display = 5 then
				edo_mux <= 0;
			else
				edo_mux <= 5;
			end if;
		end if;
		
	elsif edo_mux = 5 then
		if SEL_TRANSISTOR = '1' then
			T <= "00100000";
		else
			T <= "11011111";
		end if;	
		
		if b = '1' then		
			if num_display = 6 then
				edo_mux <= 0;
			else
				edo_mux <= 6;
			end if;
		end if;
		
	elsif edo_mux = 6 then
		if SEL_TRANSISTOR = '1' then
			T <= "01000000";
		else
			T <= "10111111";
		end if;
		
		if b = '1' then	
			if num_display = 7 then
				edo_mux <= 0;
			else
				edo_mux <= 7;
			end if;
		end if;
		
	elsif edo_mux = 7 then
		if SEL_TRANSISTOR = '1' then
			T <= "10000000";
		else
			T <= "01111111";
		end if;
		
		if b = '1' then
			edo_mux <= 0;
		end if;
	end if;
end if;
end process;

--------------------------------------------------
--------------------------------------------------


-------------------------------------------------------------------------------
-- Mquina de estados que asignal el valor a "num_aux" para ser decodificado --

process(CLK)
begin
if rising_edge(clk) then
	if edo_mux = 0 then
		num_aux <= conv_integer(DATO_DECIMAL(3 downto 0));
		punto <= puntos(0);
	elsif edo_mux = 1 then
		num_aux <= conv_integer(DATO_DECIMAL(7 downto 4));
		punto <= puntos(1);
	elsif edo_mux = 2 then
		num_aux <= conv_integer(DATO_DECIMAL(11 downto 8));
		punto <= puntos(2);
	elsif edo_mux = 3 then
		num_aux <= conv_integer(DATO_DECIMAL(15 downto 12));
		punto <= puntos(3);
	elsif edo_mux = 4 then
		num_aux <= conv_integer(DATO_DECIMAL(19 downto 16));
		punto <= puntos(4);
	elsif edo_mux = 5 then
		num_aux <= conv_integer(DATO_DECIMAL(23 downto 20));
		punto <= puntos(5);
	elsif edo_mux = 6 then
		num_aux <= conv_integer(DATO_DECIMAL(27 downto 24));
		punto <= puntos(6);
	elsif edo_mux = 7 then
		num_aux <= conv_integer(DATO_DECIMAL(31 downto 28));
		punto <= puntos(7);
	end if;
end if;
end process;

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------


-------------------------------------
----------DECODIFICADOR--------------

			 --ABCDEFG
DISPLAY <= "1001111"&punto when SEL_ANODO_CATODO = '0' and num_aux = 1 else
			  "0110000"&punto when SEL_ANODO_CATODO = '1' and num_aux = 1 else
			  "0010010"&punto when SEL_ANODO_CATODO = '0' and num_aux = 2 else
			  "1101101"&punto when SEL_ANODO_CATODO = '1' and num_aux = 2 else
			  "0000110"&punto when SEL_ANODO_CATODO = '0' and num_aux = 3 else
			  "1111001"&punto when SEL_ANODO_CATODO = '1' and num_aux = 3 else
			  "1001100"&punto when SEL_ANODO_CATODO = '0' and num_aux = 4 else
			  "0110011"&punto when SEL_ANODO_CATODO = '1' and num_aux = 4 else
			  "0100100"&punto when SEL_ANODO_CATODO = '0' and num_aux = 5 else
			  "1011011"&punto when SEL_ANODO_CATODO = '1' and num_aux = 5 else
			  "0100000"&punto when SEL_ANODO_CATODO = '0' and num_aux = 6 else
			  "1011111"&punto when SEL_ANODO_CATODO = '1' and num_aux = 6 else
			  "0001111"&punto when SEL_ANODO_CATODO = '0' and num_aux = 7 else
			  "1110000"&punto when SEL_ANODO_CATODO = '1' and num_aux = 7 else
			  "0000000"&punto when SEL_ANODO_CATODO = '0' and num_aux = 8 else
			  "1111111"&punto when SEL_ANODO_CATODO = '1' and num_aux = 8 else
			  "0000100"&punto when SEL_ANODO_CATODO = '0' and num_aux = 9 else
			  "1111011"&punto when SEL_ANODO_CATODO = '1' and num_aux = 9 else
			  "0000001"&punto when SEL_ANODO_CATODO = '0' and num_aux = 0 else
			  "1111110"&punto;
-------------------------------------
-------------------------------------

end Behavioral;

